@@ -1,5 +1,6 @@
 import gc
 import sys
+import traceback
 import warnings
 from concurrent.futures import ThreadPoolExecutor
 from functools import wraps, partial, lru_cache
@@ -27,6 +28,11 @@ TBound = TypeVar('TBound', bound='Parent')
 TConstrained = TypeVar('TConstrained', 'Parent', 'Child')
 JSONType = Union[str, int, float, bool, None, List['JSONType'], Dict[str, 'JSONType']]
 
+DummyDict = TypedDict('DummyDict', {'x': int}, total=False)
+issue_42059 = pytest.mark.xfail(bool(DummyDict.__required_keys__),
+                                reason='Fails due to upstream bug BPO-42059')
+del DummyDict
+
 
 class Parent:
     pass
@@ -70,6 +76,11 @@ def test_check_type_no_memo():
     check_type('foo', [1], List[int])
 
 
+def test_check_type_bytes():
+    pytest.raises(TypeError, check_type, 'foo', 7, bytes).\
+        match(r'type of foo must be bytes-like; got int instead')
+
+
 def test_check_type_no_memo_fail():
     pytest.raises(TypeError, check_type, 'foo', ['a'], List[int]).\
         match(r'type of foo\[0\] must be int; got str instead')
@@ -976,6 +987,14 @@ class TestTypeChecked:
 
         foo(bytearray([1]))
 
+    def test_bytearray_memoryview(self):
+        """Test that a bytearray is accepted where bytes are expected."""
+        @typechecked
+        def foo(x: bytes) -> None:
+            pass
+
+        foo(memoryview(b'foo'))
+
     def test_class_decorator(self):
         @typechecked
         class Foo:
@@ -990,11 +1009,38 @@ class TestTypeChecked:
             def method(self) -> int:
                 return 'foo'
 
+            @property
+            def prop(self) -> int:
+                return 'foo'
+
+            @property
+            def prop2(self) -> int:
+                return 'foo'
+
+            @prop2.setter
+            def prop2(self, value: int) -> None:
+                pass
+
         pattern = 'type of the return value must be int; got str instead'
         pytest.raises(TypeError, Foo.staticmethod).match(pattern)
         pytest.raises(TypeError, Foo.classmethod).match(pattern)
         pytest.raises(TypeError, Foo().method).match(pattern)
 
+        with pytest.raises(TypeError) as raises:
+            Foo().prop
+
+        assert raises.value.args[0] == pattern
+
+        with pytest.raises(TypeError) as raises:
+            Foo().prop2
+
+        assert raises.value.args[0] == pattern
+
+        with pytest.raises(TypeError) as raises:
+            Foo().prop2 = 'foo'
+
+        assert raises.value.args[0] == 'type of argument "value" must be int; got str instead'
+
     @pytest.mark.parametrize('annotation', [
         Generator[int, str, List[str]],
         Generator,
@@ -1138,6 +1184,37 @@ class TestTypeChecked:
         assert Child.foo('bar') == 'Child'
         pytest.raises(TypeError, Child.foo, 1)
 
+    def test_class_property(self):
+        @typechecked
+        class Foo:
+            def __init__(self) -> None:
+                self.foo = 'foo'
+
+            @property
+            def prop(self) -> int:
+                """My property."""
+                return 4
+
+            @property
+            def prop2(self) -> str:
+                return self.foo
+
+            @prop2.setter
+            def prop2(self, value: str) -> None:
+                self.foo = value
+
+        assert Foo.__dict__["prop"].__doc__.strip() == "My property."
+        f = Foo()
+        assert f.prop == 4
+        assert f.prop2 == 'foo'
+        f.prop2 = 'bar'
+        assert f.prop2 == 'bar'
+
+        with pytest.raises(TypeError) as raises:
+            f.prop2 = 3
+
+        assert raises.value.args[0] == 'type of argument "value" must be str; got int instead'
+
     def test_decorator_factory_no_annotations(self):
         class CallableClass:
             def __call__(self):
@@ -1205,12 +1282,15 @@ class TestTypeChecked:
                   r'List, Dict\); got dict instead')
 
     def test_literal(self):
+        from http import HTTPStatus
+
         @typechecked
-        def foo(a: Literal[1, 6, 8]):
+        def foo(a: Literal[1, True, 'x', b'y', HTTPStatus.ACCEPTED]):
             pass
 
-        foo(6)
-        pytest.raises(TypeError, foo, 4).match(r'must be one of \(1, 6, 8\); got 4 instead$')
+        foo(HTTPStatus.ACCEPTED)
+        pytest.raises(TypeError, foo, 4).match(r"must be one of \(1, True, 'x', b'y', "
+                                               r"<HTTPStatus.ACCEPTED: 202>\); got 4 instead$")
 
     def test_literal_union(self):
         @typechecked
@@ -1222,18 +1302,38 @@ class TestTypeChecked:
             match(r'must be one of \(str, typing(_extensions)?.Literal\[1, 6, 8\]\); '
                   r'got int instead$')
 
+    def test_literal_nested(self):
+        @typechecked
+        def foo(a: Literal[1, Literal['x', 'a', Literal['z']], 6, 8]):
+            pass
+
+        foo('z')
+        pytest.raises(TypeError, foo, 4).match(r"must be one of \(1, 'x', 'a', 'z', 6, 8\); "
+                                               r"got 4 instead$")
+
+    def test_literal_illegal_value(self):
+        @typechecked
+        def foo(a: Literal[1, 1.1]):
+            pass
+
+        pytest.raises(TypeError, foo, 4).match(r"Illegal literal value: 1.1$")
+
     @pytest.mark.parametrize('value, total, error_re', [
-        ({'x': 6, 'y': 'foo'}, True, None),
-        ({'y': 'foo'}, True, r'required key\(s\) \("x"\) missing from argument "arg"'),
-        ({'x': 6, 'y': 3}, True,
-         'type of dict item "y" for argument "arg" must be str; got int instead'),
-        ({'x': 6}, True, r'required key\(s\) \("y"\) missing from argument "arg"'),
-        ({'x': 6}, False, None),
-        ({'x': 'abc'}, False,
-         'type of dict item "x" for argument "arg" must be int; got str instead'),
-        ({'x': 6, 'foo': 'abc'}, False, r'extra key\(s\) \("foo"\) in argument "arg"'),
-    ], ids=['correct', 'missing_x', 'wrong_y', 'missing_y_error', 'missing_y_ok', 'wrong_x',
-            'unknown_key'])
+        pytest.param({'x': 6, 'y': 'foo'}, True, None, id='correct'),
+        pytest.param({'y': 'foo'}, True, r'required key\(s\) \("x"\) missing from argument "arg"',
+                     id='missing_x'),
+        pytest.param({'x': 6, 'y': 3}, True,
+                     'type of dict item "y" for argument "arg" must be str; got int instead',
+                     id='wrong_y'),
+        pytest.param({'x': 6}, True, r'required key\(s\) \("y"\) missing from argument "arg"',
+                     id='missing_y_error'),
+        pytest.param({'x': 6}, False, None, id='missing_y_ok', marks=[issue_42059]),
+        pytest.param({'x': 'abc'}, False,
+                     'type of dict item "x" for argument "arg" must be int; got str instead',
+                     id='wrong_x', marks=[issue_42059]),
+        pytest.param({'x': 6, 'foo': 'abc'}, False, r'extra key\(s\) \("foo"\) in argument "arg"',
+                     id='unknown_key')
+    ])
     def test_typed_dict(self, value, total, error_re):
         DummyDict = TypedDict('DummyDict', {'x': int, 'y': str}, total=total)
 
@@ -1423,3 +1523,18 @@ class TestTypeChecker:
         assert len(record) == 1
         assert str(record[0].message).startswith("Replaced forward declaration 'OrderedDict' in")
         assert unresolvable_annotation.__annotations__['x'] is collections.OrderedDict
+
+
+class TestTracebacks:
+    def test_short_tracebacks(self):
+        def foo(a: Callable[..., int]):
+            assert check_argument_types()
+
+        try:
+            foo(1)
+        except TypeError:
+            _, _, tb = sys.exc_info()
+            parts = traceback.extract_tb(tb)
+            typeguard_lines = [part for part in parts
+                               if part.filename.endswith("typeguard/__init__.py")]
+            assert len(typeguard_lines) == 1
